/**
 * @author ke.chang@rrze.uni-erlangen.de
 */

// navTree operating object. requires jQuery!

var navTreeOper = {
	_cloneObject: function(obj) {
		if(typeof obj !== 'object' || obj == null)
			return obj;
		
		var c = obj instanceof Array ? [] : {};
		for(var i in obj) {
			var prop = obj[i];
			if(typeof prop == 'object') {
				if(prop instanceof Array) {
					c[i] = [];
					for(var j = 0; j < prop.length; j++) {
						if(typeof prop[j] != 'object') {
							c[i].push(prop[j]);
						} else {
							c[i].push(this._cloneObject(prop[j]));
						}
					}
				} else {
					c[i] = this._cloneObject(prop);
				}
			} else {
				c[i] = prop;
			}
		}
		return c;
	},
	
	_jsonObject: {},
	_jsonString: "",
	
	// 3 parts json: A, Z, S
	_jsonNavNormal: [],
	_jsonNavExtra: [],
	_jsonNavSpec: [],
	
	_dirHtmlNormal: "",
	_dirHtmlExtra: "",
	_dirHtmlSpec: "",
	
	// current objects
	_currentKey: "",
	_currentNode: {},
	_currentNodeGroup: [],
	
	// oper flags
	_canAddSibling: false,
	_canAddChild: false,
	_canCut: false,
	_canRemove: false,
	_canPasteAsSibling: false,
	_canPasteAsChild: false,
	
	_clipBoard: {
		depth: 0,
		data: {}
	},
	
	_treeDepth: 1,
	
	_undoStack: [],
	
	// html containers
	dirHtmlPanelId: "",
	dirDivIdNormal: "",
	dirDivIdExtra: "",
	dirDivIdSpec: "",
	debugInfoPanelId: "",
	
	nodeEventFuncName: "",
	
	setCurrentKey: function(ckey) {
		this._currentKey = ckey;
	},
	
	setCurrentNode: function(cnode) {
		this._currentNode = cnode;
	},
	
	setCurrentNodeGroup: function(groupType) {
		switch(groupType) {
			case "Z":
				this._currentNodeGroup = this._jsonNavExtra;
				break;
			case "S":
				this._currentNodeGroup = this._jsonNavSpec;
				break;
			default:
				alert("setCurrentNodeGroup error!");
				break;
		}
	},
	
	getCurrentNode: function() {
		return this._currentNode;
	},
	
	_hasClipData: function() {
		if((this._clipBoard.depth > 0) && (this._clipBoard.data != null)) {
			return true;
		} else {
			return false;
		}
	},
	
	_getNodeType: function(nodeKey) {
		var firstLetter = nodeKey.substr(0, 1);
		if((firstLetter.toUpperCase() == "S") || (firstLetter.toUpperCase() == "Z")) {
			return firstLetter;
		} else {
			return "A";
		}
	},
	
	_genDirHtmlNormal: function(jsonData) {
		if(jsonData.length < 1) {
			return;
		}
		this._dirHtmlNormal += "<ul>";
		for(var i = 0; i < jsonData.length; i++) {
			this._dirHtmlNormal += "<li><span id=\"" + jsonData[i].key + "\">" + jsonData[i].title + "</span>";
			if(jsonData[i].child.length > 0) {
				this._genDirHtmlNormal(jsonData[i].child); // recursive
			}
			this._dirHtmlNormal += "</li>";
		}
		this._dirHtmlNormal += "</ul>";
	},
	
	_genDirHtmlExtra: function(jsonData) {
		if(jsonData.length < 1) {
			return;
		}
		this._dirHtmlExtra += "<ul>";
		for(var j = 0; j < jsonData.length; j++) {
			this._dirHtmlExtra += "<li><span id=\"" + jsonData[j].key + "\">" + jsonData[j].title + "</span></li>";
		}
		this._dirHtmlExtra += "</ul>";
	},
	
	_genDirHtmlSpec: function(jsonData) {
		if(jsonData.length < 1) {
			return;
		}
		this._dirHtmlSpec += "<ul>";
		for(var k = 0; k < jsonData.length; k++) {
			this._dirHtmlSpec += "<li><span id=\"" + jsonData[k].key + "\">" + jsonData[k].title + "</span></li>";
		}
		this._dirHtmlSpec += "</ul>";
	},
	
	init: function() {
		$("#" + this.dirDivIdNormal).empty();
		
		this._dirHtmlNormal = "";
		this._dirHtmlExtra = "";
		this._dirHtmlSpec = "";
		
		this._currentKey = "";
		this._currentNode = {};
		this._currentNodeGroup = [];
		
		this._canAddSibling = false;
		this._canAddChild = false;
		this._canCut = false;
		this._canPasteAsSibling = false;
		this._canPasteAsChild = false;
		this._canRemove = false;
		
		$("#" + this.debugInfoPanelId).text("Ready.");
		
		this.updateButtons();
	},
	
	locateNode: function(nodeKey) {
		var ntyp = this._getNodeType(nodeKey);
		if(ntyp != "A") { // S and Z pages, are all on root (no hierarchy)
			if(ntyp == "Z")
				this._currentNodeGroup = this._jsonNavExtra;
			else
				this._currentNodeGroup = this._jsonNavSpec;
			
			for(var j = 0; j < this._currentNodeGroup.length; j++) {
				if(this._currentNodeGroup[j].key == nodeKey) {
					this._canAddSibling = true;
					this._canAddChild = false;
/*					this._canCut = false;
					this._canPasteAsSibling = false;
					this._canPasteAsChild = false;*/
					this._canRemove = true;
					
					return this._currentNodeGroup[j];
				}
			}
		} else { // normal pages, but be ware of U
			if(nodeKey == "U") { // special
				this._currentNodeGroup = this._jsonNavNormal;
				
				this._canAddSibling = true;
				this._canAddChild = false;
/*				this._canCut = false;
				if(this._hasClipData()) {
					this._canPasteAsSibling = true;
				}
				this._canPasteAsChild = false;*/
				this._canRemove = false;
				
				return this._jsonNavNormal[0];
			} else { // normal node, with hierarchy
				var arrKeys = nodeKey.split("-");
				if(arrKeys.length == 1) { // root level: Ax
					this._currentNodeGroup = this._jsonNavNormal;
					
					this._canAddSibling = true;
					this._canAddChild = true;
/*					this._canCut = true;
					if(this._hasClipData()) {
						this._canPasteAsSibling = true;
						this._canPasteAsChild = true;
					}*/
					this._canRemove = true;
					
					var arrRootLevelKeys = /A(\d+)/.exec(nodeKey);
					var rootIdx = arrRootLevelKeys[1] * 1; // convert to int
					return this._jsonNavNormal[rootIdx];
				} else { // most complicated part
					// >_<...
					this._canAddSibling = true;
					this._canAddChild = true;
/*					this._canCut = true;
					if(this._hasClipData()) {
						this._canPasteAsSibling = true;
						this._canPasteAsChild = true;
					}*/
					this._canRemove = true;
					
					var idx0 = arrKeys[0].substr(1) * 1;
					var idx1 = arrKeys[arrKeys.length - 1].substr(1) * 1;
					this._currentNodeGroup = this._jsonNavNormal[idx0].child;
					for(var i = 2; i < arrKeys.length; i++) { // start from 3rd...
						idx0 = arrKeys[i - 1].substr(1) * 1;
						this._currentNodeGroup = this._currentNodeGroup[idx0 - 1].child; // root level, number = index, deeper ones must -1
					}
					return this._currentNodeGroup[idx1 - 1];
				}
			}
		}
	},
	
	updateButtons: function() {
		$("#btnAddSibling").attr("disabled", !this._canAddSibling);
		$("#btnAddChild").attr("disabled", !this._canAddChild);
		$("#btnCut").attr("disabled", !this._canCut);
		$("#btnPaste0").attr("disabled", !this._canPasteAsSibling);
		$("#btnPaste1").attr("disabled", !this._canPasteAsChild);
		$("#btnRemove").attr("disabled", !this._canRemove);
	},
	
	refreshNavTree: function(jsonTreeData) {
		this.init();
		$("#" + this.debugInfoPanelId).text("Tree-Data loaded...");
		
		this._jsonObject = jsonTreeData;
		this._jsonNavNormal = jsonTreeData.A;
		this._jsonNavExtra = jsonTreeData.Z;
		this._jsonNavSpec = jsonTreeData.S;
		
		this._reIndex(this._jsonNavNormal);
		this._reIndex(this._jsonNavExtra);
		this._reIndex(this._jsonNavSpec);
		
		this.addPathInfo(this._jsonNavNormal);
		this.addPathInfo(this._jsonNavExtra);
		this.addPathInfo(this._jsonNavSpec);
		
		this._genDirHtmlNormal(this._jsonNavNormal);
		this._genDirHtmlExtra(this._jsonNavExtra);
		this._genDirHtmlSpec(this._jsonNavSpec);
		
		$("#" + this.dirDivIdNormal).html(this._dirHtmlNormal);
		$("#" + this.dirDivIdExtra).html(this._dirHtmlExtra);
		if(this._dirHtmlExtra == "") {
			$("#btnNewExtraNode").show();
		} else {
			$("#btnNewExtraNode").hide();
		}
		$("#" + this.dirDivIdSpec).html(this._dirHtmlSpec);
		if(this._dirHtmlSpec == "") {
			$("#btnNewSpecNode").show();
		} else {
			$("#btnNewSpecNode").hide();
		}
		
		eval(this.nodeEventFuncName);
	},
	
	_hasDupTitle: function(title, nodes) {
		for(var i = 0, li = nodes.length; i < li; i++) {
			if(nodes[i].title.toLowerCase() == title.toLowerCase()) {
				alert("Error: There are duplicate titles, please retry!");
				return true;
			}
		}
		return false;
	},
	
	_reIndex: function(nodes) {
		if(nodes.length < 1) {
			return;
		}
		var testKey = nodes[0].key;
		var typ = this._getNodeType(testKey);
		if(typ == "A") { // normal nodes
			// reindex normal nodes, recursive
			if(testKey.substr(0, 1) == "U") { // root level
				for(var i = 1; i < nodes.length; i++) {
					nodes[i].key = "A" + i.toString();
					if(nodes[i].child.length > 0) {
						var oldChildKey0 = nodes[i].child[0].key;
						nodes[i].child[0].key = oldChildKey0.replace(new RegExp("A(\\d+)"), "A" + i.toString());
						this._reIndex(nodes[i].child);
					}
				}
			} else { // Ax-Bx-...
				// ...
				var arrKeys = testKey.split("-");
				var lastKeyComp = String.fromCharCode(64 + arrKeys.length);
				var fixedPart = testKey.substring(0, testKey.lastIndexOf("-")) + "-";
				for(var k = 0; k < nodes.length; k++) {
					nodes[k].key = fixedPart + lastKeyComp + (k + 1).toString();
					if(nodes[k].child.length > 0) {
						var oldChildKey1 = nodes[k].child[0].key;
						nodes[k].child[0].key = oldChildKey1.replace(oldChildKey1.substring(0, oldChildKey1.lastIndexOf("-")), nodes[k].key);
						this._reIndex(nodes[k].child);
					}
				}
			}
		} else { // extra nodes or spec nodes
			for(var j = 0; j < nodes.length; j++) {
				nodes[j].key = typ + (j + 1).toString();
			}
		}
	},
	
	addSibling: function(titleText) {
		if(this._jsonObject == null) {
			alert("ERROR: _jsonObject == null");
			return;
		}
		
		if(this._hasDupTitle(titleText, this._currentNodeGroup)) {
			return;
		}
		
		var newNodeObj = {
			"key": "",
			"title": titleText,
			"child": [],
			"alias": "",
			"accesskey": "",
			"email": "",
			"path": "",
			"quicklink": false,
			"displaylink": false,
			"url": "",
			"info_text": ""
		};
		
		var path = this._currentKey;
		var arrPath = path.split("-");
		switch(arrPath.length) {
			case 1:
				var idx = 0;
				switch(path.substr(0, 1)) {
					case "U":
						newNodeObj.key = "A1";
						idx = 1;
						break;
					case "Z":
						var keysZ = /Z(\d+)/.exec(path);
						var zx = keysZ[1] * 1;
						newNodeObj.key = "Z" + (zx + 1).toString();
						idx = zx;
						break;
					case "S":
						var keysS = /S(\d+)/.exec(path);
						var sx = keysS[1] * 1;
						newNodeObj.key = "S" + (sx + 1).toString();
						idx = sx;
						break;
					case "A":
						var keysA = /A(\d+)/.exec(path);
						var ax = keysA[1] * 1;
						newNodeObj.key = "A" + (ax + 1).toString();
						idx = ax + 1;
						break;
					default:
						alert("Errr...");
						break;
				}
				this._currentNodeGroup.splice(idx, 0, newNodeObj);
				this._reIndex(this._currentNodeGroup);
				break;
			default:
				var arrlastPart = /([A-Z])(\d+)/.exec(arrPath[arrPath.length - 1]);
				var idx1 = arrlastPart[2] * 1;
				this._currentNodeGroup.splice(idx1, 0, newNodeObj);
				this._reIndex(this._currentNodeGroup);
				break;
		}
		this.refreshNavTree(this._jsonObject);
		return newNodeObj;
	},
	
	addChild: function(titleText) {
		if(this._jsonObject == null) {
			alert("ERROR: _jsonObject == null");
			return;
		}
		
		if(this._hasDupTitle(titleText, this._currentNode.child)) {
			return;
		}
		
		var newChildObj = {
			"key": "",
			"title": titleText,
			"child": [],
			"alias": "",
			"accesskey": "",
			"email": "",
			"path": "",
			"quicklink": false,
			"displaylink": false,
			"url": "",
			"info_text": ""
		};
		
		var path = this._currentKey;
		var arrPath = path.split("-");
		switch(arrPath.length) {
			case 1:
				switch(path.substr(0, 1)) {
					case "A":
						if(this._currentNode.child.length < 1) {
							var arrRootKeys = /A(\d+)/.exec(path);
							var ax = arrRootKeys[0] + "";
							newChildObj.key = ax + "-B1";
						}
//						this._currentNode.child.unshift(newChildObj); // new child should be at place 0
						this._currentNode.child.push(newChildObj); // new child should be at the last place
						this._reIndex(this._currentNode.child);
						break;
				}
				break;
			default:
				var arrLastLetter = /([A-Z])/.exec(arrPath[arrPath.length - 1]);
				var lastLetter = arrLastLetter[1];
				if(lastLetter == "Z") {
					alert("No more Child-Node allowed!");
					return;
				}
				if(this._currentNode.child.length < 1) {
					var newLetter = String.fromCharCode(lastLetter.charCodeAt(0) + 1);
					newChildObj.key = path + "-" + newLetter + "1";
				}
//				this._currentNode.child.unshift(newChildObj);
				this._currentNode.child.push(newChildObj);
				this._reIndex(this._currentNode.child);
				break;
		}
		this.refreshNavTree(this._jsonObject);
		return newChildObj;
	},
	
	remove: function() {
		if(this._jsonObject == null) {
			alert("ERROR: _jsonObject == null");
			return;
		}
		for(var i = 0; i < this._currentNodeGroup.length; i++) {
			if(this._currentNodeGroup[i].key == this._currentKey) {
				this._currentNodeGroup.splice(i, 1);
				break;
			}
		}
		this._reIndex(this._currentNodeGroup);
		this.refreshNavTree(this._jsonObject);
	},
	
	_getTreeDepth: function(tree) {
		if(tree.length > 0) {
			this._treeDepth++;
		}
		for(var t = 0; t < tree.length; t++) {
			if(tree[t].child.length > 0) {
				this._getTreeDepth(tree[t].child);
			}
		}
	},
	
	cut: function() {
		// tbd...
		this._treeDepth = 1;
		this._getTreeDepth(this._currentNode.child);
		
		this._clipBoard.depth = this._treeDepth;
		this._clipBoard.data = this._currentNode;
		
		this.remove();
	},
	
	pasteAsSibling: function() {
		// almost the same as addSibling()
		if(this._jsonObject == null) {
			alert("ERROR: _jsonObject == null");
			return;
		}
		
		var newNodeObj = this._cloneObject(this._clipBoard.data);
		
		if(this._hasDupTitle(newNodeObj.title, this._currentNodeGroup)) {
			return;
		}
		
		var path = this._currentKey;
		var arrPath = path.split("-");
		switch(arrPath.length) {
			case 1:
				var idx = 0;
				switch(path.substr(0, 1)) {
					case "U":
						newNodeObj.key = "A1";
						idx = 1;
						break;
					case "Z":
						var keysZ = /Z(\d+)/.exec(path);
						var zx = keysZ[1] * 1;
						newNodeObj.key = "Z" + (zx + 1).toString();
						idx = zx;
						break;
					case "S":
						var keysS = /S(\d+)/.exec(path);
						var sx = keysS[1] * 1;
						newNodeObj.key = "S" + (sx + 1).toString();
						idx = sx;
						break;
					case "A":
						var keysA = /A(\d+)/.exec(path);
						var ax = keysA[1] * 1;
						newNodeObj.key = "A" + (ax + 1).toString();
						idx = ax + 1;
						break;
					default:
						alert("Errr...");
						break;
				}
				this._currentNodeGroup.splice(idx, 0, newNodeObj);
				this._reIndex(this._currentNodeGroup);
				break;
			default:
				if((path.length + this._clipBoard.depth - 1) > 26) {
					// level too deep
					alert("Level too deep!");
					return;
				}
				
				var arrlastPart = /([A-Z])(\d+)/.exec(arrPath[arrPath.length - 1]);
				var idx1 = arrlastPart[2] * 1;
				this._currentNodeGroup.splice(idx1, 0, newNodeObj);
				this._reIndex(this._currentNodeGroup);
				break;
		}
		this.refreshNavTree(this._jsonObject);
	},
	
	pasteAsChild: function() {
		if(this._jsonObject == null) {
			alert("ERROR: _jsonObject == null");
			return;
		}
		
		var newChildObj = this._cloneObject(this._clipBoard.data);
		
		if(this._hasDupTitle(newChildObj.title, this._currentNode.child)) {
			return;
		}
		
		var path = this._currentKey;
		var arrPath = path.split("-");
		switch(arrPath.length) {
			case 1:
				switch(path.substr(0, 1)) {
					case "A":
						var arrRootKeys = /A(\d+)/.exec(path);
						var ax = arrRootKeys[0] + "";
						newChildObj.key = ax + "-B1";
						this._currentNode.child.unshift(newChildObj); // new child should be at place 0
						this._reIndex(this._currentNode.child);
						break;
				}
				break;
			default:
				if((path.length + this._clipBoard.depth) > 26) {
					alert("Level too deep!");
					return;
				}
				
				var arrLastLetter = /([A-Z])/.exec(arrPath[arrPath.length - 1]);
				var lastLetter = arrLastLetter[1];
				if(lastLetter == "Z") {
					alert("No more Child-Node allowed!");
					return;
				}
				var newLetter = String.fromCharCode(lastLetter.charCodeAt(0) + 1);
				newChildObj.key = path + "-" + newLetter + "1";
				this._currentNode.child.unshift(newChildObj);
				this._reIndex(this._currentNode.child);
				break;
		}
		this.refreshNavTree(this._jsonObject);
	},
	
	getJSONObject: function() {
		return this._jsonObject;
	},
	
	getFileName: function(node) {
		if(node.title == null) {
			return;
		}
		
		if(node.alias != "") {
			return node.alias;
		}
		
		var fname = node.title;
		fname = fname.toLowerCase();
		
//		fname = fname.replace("ß", "ss");
//		fname = fname.replace("ä", "ae");
//		fname = fname.replace("ö", "oe");
//		fname = fname.replace("ü", "ue");
		
		fname = fname.replace(/\u00df/, "ss");
		fname = fname.replace(/\u00e4/, "ae");
		fname = fname.replace(/\u00f6/, "oe");
		fname = fname.replace(/\u00fc/, "ue");
		fname = fname.replace(/([^\w.\-_])/g, "-");
		fname = fname.replace(/-{2,}/, "-");
		
		return fname;
	},
	
	getPathInfo: function(node) {
		if(node.key == null) {
			return "";
		}
		
		var pkey = node.key;
		var fname = "";
		if(this._getNodeType(pkey) == "A") {
			var arrPKey = pkey.split("-");
			if(arrPKey.length == 1) { // root level
				if(node.child.length > 0) {
					fname = "/" + this.getFileName(node) + "/index.shtml";
				} else {
					fname = "/" + this.getFileName(node) + ".shtml";
				}
			} else {
				var idx0 = arrPKey[0].substr(1) * 1;
				var idx1 = arrPKey[arrPKey.length - 1].substr(1) * 1;
				var locatorGroup = this._jsonNavNormal[idx0].child;
				fname = "/" + this.getFileName(this._jsonNavNormal[idx0]);
				for(var i = 2; i < arrPKey.length; i++) { // start from 3rd...
					idx0 = arrPKey[i - 1].substr(1) * 1;
					fname += "/" + this.getFileName(locatorGroup[idx0 - 1]);
					locatorGroup = locatorGroup[idx0 - 1].child; // root level, number = index, deeper ones must -1
				}
				fname += "/" + this.getFileName(locatorGroup[idx1 - 1]);
				if(node.child.length > 0) {
					fname += "/index.shtml";
				} else {
					fname += ".shtml";
				}
			}
		} else { // Z und S
			fname = "/" + this.getFileName(node) + ".shtml";
		}
		return fname;
	},
	
	addPathInfo: function(tree) {
		for(var i = 0; i < tree.length; i++) {
			tree[i].path = this.getPathInfo(tree[i]);
			if(tree[i].child.length > 0) {
				this.addPathInfo(tree[i].child);
			}
		}
	}
};
